import { RegisteredFunction } from './function-registry';
import { FrameworkOptions } from './options';
import { RouteDefinition } from './types';
export interface IRouter {
    /**
     * 根据 path 查找路由信息，返回匹配到的原始路由定义
     * @param path 路径，不带有 query 和 hash
     * @returns 路由信息
     */
    findRoute(path: string): RouteDefinition | null;
}
export declare class Router {
    private routes;
    private router;
    constructor(options: FrameworkOptions);
    private addRoute;
    /**
     * 根据 path 查找路由信息，返回匹配到的原始路由定义
     * @param path 路径
     * @returns 路由信息
     */
    findRoute(path: string): RouteDefinition | null;
}
/**
 * 路由匹配配置项
 */
type TRouteFunctionOptions = {
    /**
     * 是否是多函数模式
     */
    isMultiFunctions?: boolean;
    /**
     * 路由定义
     */
    routes?: RouteDefinition[];
};
/**
 * 路由匹配结果
 */
type TRouteFunctionResult = {
    /**
     * 匹配到的路由定义
     */
    routeDefinition?: RouteDefinition;
    /**
     * 匹配到的函数
     */
    handleFunction?: RegisteredFunction;
};
/**
 * 根据 path 查找匹配的路由。
 *
 * 路由规则：
 * 1. 每一条路由的 path 匹配均为前缀匹配，即 /aa 可以匹配 /aa/bb ，/aa/bb/cc等等
 * 2. 如有多条相同 path 规则，先声明的优先匹配
 * 3. path 规则中带 / 的和不带 / 的路由视为同一条路由，如 /aa 和 /aa/ 认为是相同的，前面的规则会覆盖后面的，即以第一条声明的为准
 *     a. 如果请求时 path 为空，将会被视作 / ，会命中配置 / 的规则（如果未配置 / ，则 404）
 * 4. 如有多条规则匹配，将会命中最长匹配的规则，如 /a -> a , /a/b -> b ，path /a/b/c将会匹配至 b
 * 5. 匹配以 / 分隔为最小的一段，单个段内不会模糊匹配，如/aaa 不会匹配 /a -> a 规则
 *
 * @param router IRouter 实例
 * @param path 路径，不带有 query 和 hash
 * @param option 配置项
 * @returns
 */
export declare function routeFunction(router: IRouter, path: string, option: TRouteFunctionOptions): TRouteFunctionResult;
export {};
//# sourceMappingURL=router.d.ts.map