import { BaseContext } from '@cloudbase/functions-typings';
import { ContextInjected, ExtendedContext, sse } from '@cloudbase/functions-typings';
import { HTTPContext } from '@cloudbase/functions-typings';
import Koa, { Middleware } from 'koa';
import { IRouter } from './router';
export { EventFunction, FormEvent, HandlerFunction, IntegrationResponse } from '@cloudbase/functions-typings';
export { ContextInjected } from '@cloudbase/functions-typings';
export { HttpBasis, HttpMethod } from '@cloudbase/functions-typings';
export { sse, ws } from '@cloudbase/functions-typings';
export { BufferLike } from '@cloudbase/functions-typings';
/**
 * Supported signature types.
 */
export declare const SignatureType: readonly ["event"];
/**
 * Union type of all valid function SignatureType values.
 */
export type SignatureType = (typeof SignatureType)[number];
/**
 * ErrorResult 包含错误的 Result
 */
export type ErrorResult = {
    code: string;
    message: string;
};
/**
 * IExtendedState 扩展的请求状态信息
 */
export interface IExtendedState extends BaseContext {
    event?: unknown;
    contextInjected?: ContextInjected;
    codeInfo: UserCodePkgInfo;
    /**
     * 是否超时
     */
    isTimeout?: boolean;
    /**
     * 用户代码抛出的错误
     */
    error?: Error;
    errorResult?: ErrorResult;
    /**
     * 用户代码耗时
     */
    userCodeTimeCost?: number;
    /**
     * 匹配的路由，仅在多函数模式下有效
     */
    matchedRoute?: RouteDefinition;
}
/**
 * IExtendedContext 扩展的请求上下文
*/
export interface IExtendedContext extends HTTPContext, ExtendedContext {
    rawBody?: Buffer;
    sse?: () => sse.IServerSentEvent;
}
/**
 * 包含路由的 Koa 类型
 */
export type TKoaApp = InstanceType<typeof Koa<IExtendedState, IExtendedContext>> & {
    router: IRouter;
};
export type KoaMiddleware = Middleware<IExtendedState, IExtendedContext>;
export type UserCodePkgInfo = {
    name?: string;
    version?: string;
};
export declare enum LogType {
    ACCESSLOG = "accesslog",
    USERCODELOG = "usercodelog"
}
export interface ILog {
    '@timestamp': string;
    logType: LogType;
}
export interface IUserCodeLogItem {
    '@timestamp'?: string;
    caller?: string;
    stacktraces?: string;
    level?: string;
    content: string | Uint8Array;
}
export interface IUserCodeLog {
    '@timestamp': string;
    logType: LogType;
    eventId: string;
    scene?: 'logBeforeReturn' | 'logAfterReturn';
    logs: IUserCodeLogItem[];
}
/**
 * FunctionDefinition 函数声明
 */
export interface FunctionDefinition {
    /**
     * 函数名称
     */
    name: string;
    /**
     * 函数目录
     */
    directory: string;
    /**
     * 函数入口文件
     */
    source?: string;
    /**
     * 目标函数名称
     */
    target?: string;
    /**
     * 匹配的路径，前缀匹配
     */
    triggerPath?: string;
}
/**
 * RouteDefinition 路由声明
 */
export interface RouteDefinition {
    /**
     * 目标函数名称
     */
    functionName: string;
    /**
     * 匹配的路径，前缀匹配
     */
    path?: string;
}
/**
 * MultiFnConfig 多函数配置
 */
export interface MultiFnConfig {
    /**
     * 函数根目录
     */
    functionsRoot: string;
    /**
     * 函数声明
     */
    functions: FunctionDefinition[];
    /**
     * 路由声明
     */
    routes: RouteDefinition[];
}
//# sourceMappingURL=types.d.ts.map